// Testing basic crud functionality

// Testing readAll() method

var assert = require('assert'),
    vows = require('vows'),
    databank = require('../databank'),
    Databank = databank.Databank,
    AlreadyExistsError = databank.AlreadyExistsError,
    driverParams = require('./driver-params').driverParams;

var doubleCreateContext = function(driver, params) {

    var context = {};

    context["When we create a " + driver + " databank"] = {

        topic: function() {
            params.schema = {
                test: {
                    pkey: 'number'
                }
            };
            return Databank.get(driver, params);
        },

        'we can connect to it': {
            topic: function(bank) {
                bank.connect(params, this.callback);
            },
            'without an error': function(err) {
                assert.ifError(err);
            },
            'and we can create an item': {
                topic: function(bank) {
                    bank.create('test', 5, {'pass': true, 'iters': 42}, this.callback); 
                },
                'without an error': function(err, value) {
                    assert.ifError(err);
                    assert.isObject(value);
                    assert.equal(value.pass, true);
                    assert.equal(value.iters, 42);
                },
                'and create it again': {
                    topic: function(created, bank) {
                        bank.create('test', 5, {'pass': true, 'iters': 42}, this.callback); 
                    },
                    'with an AlreadyExistsError': function(err, value) {
                        assert.isObject(err);
                        assert.instanceOf(err, AlreadyExistsError);
                        assert.isNull(value);
                    },
                    'and delete it': {
                        topic: function(failed, created, bank) {
                            bank.del('test', 5, this.callback);
                        },
                        'without an error': function(err) {
                            assert.ifError(err);
                        },
                        'and we can disconnect': {
                            topic: function(failed, created, bank) {
                                bank.disconnect(this.callback);
                            },
                            'without an error': function(err) {
                                assert.ifError(err);
                            }
                        }
                    }
                }
            }
        }
    };

    return context;
};

var suite = vows.describe('double-create exception'),
    driver = null;

for (driver in driverParams) {
    suite.addBatch(doubleCreateContext(driver, driverParams[driver]));
}

suite.export(module);
